/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.dods;

// Kelp imports
import org.enhydra.kelp.common.event.WriteEvent;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.swing.AddinInnerPanel;

// ToolBox imports
import org.enhydra.tool.common.DataValidationException;
import org.enhydra.tool.common.ButtonPanel;
import org.enhydra.tool.common.ToolException;

// Standard imports
import java.awt.*;
import java.lang.ref.WeakReference;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;

//
public class DodsInnerPanel extends AddinInnerPanel {

    private int               showIndex = 0;
    private JTabbedPane       tab = null;
    private BorderLayout      layoutMain = null;
    private GeneralPanel      generalPanel = null;
    private WeakReference     backRef = null;
    private WeakReference     nextRef = null;

    public DodsInnerPanel() {
        super();
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // override AddinInnerPanel
    public void read(OtterNode node) {
        super.read(node);
        DodsStep[] steps = new DodsStep[0];

        steps = getSteps();
        for (int i = 0; i < steps.length; i++) {
            steps[i].read(getProject());
        }

    }

    // override AddinInnerPanel
    public void write(OtterNode node) throws DataValidationException {
        super.write(node);

        generalPanel.write(getProject());
    }

    // override AddinInnerPanel
    public void initPreferredSize() {
        Dimension d = new Dimension();

        d.height = 500;
        d.width = 600;
        setPreferredSize(d);
    }

    // override AddinInnerPanel
    public void clearAll() {
        super.clearAll();
        if (backRef != null) {
            backRef.clear();
        }
        if (nextRef != null) {
            nextRef.clear();
        }
        generalPanel.clearAll();
        backRef = null;
        nextRef = null;
        generalPanel = null;
    }

    //
    private void refresh() {
        Component back = null;
        Component next = null;
        boolean   backEnable = true;
        boolean   nextEnable = true;

        int index = tab.getSelectedIndex();

        if (index == 0) {
            backEnable = false;
        } else if (index == (tab.getTabCount() - 1)) {
            nextEnable = false;
        }
        if (back != null) {
            back.setEnabled(backEnable);
        }
        if (next != null) {
            next.setEnabled(nextEnable);
        }
        generalPanel.refresh();
    }

    protected void back() {
        int index = tab.getSelectedIndex();

        if (index > 0) {
            showIndex = --index;
            tab.setSelectedIndex(showIndex);
        }
        refresh();
    }

    private WriteListener getWriteListener() {
        return generalPanel.getWriteListener();
    }

    protected void generate(CoreDodsTool tool) {

       if (getProject() != null) {

            Generator generate = null;
            try {
                WriteEvent clear;

                clear = new WriteEvent(this, WriteEvent.CLEAR, new String());
                trySave();
                generate = new Generator(getWriteListener());
                generate.setProject(getProject());
                generate.setEcho(true);
                generate.addProgressListener(tool.getProgressMeter());
                tool.getProgressMeter().addCancelListener(generate);
                getWriteListener().onWrite(clear);

                tool.openProgress();

                generate.setOwner(getWindow());
                generate.setDoneDialog(true);

                generate.build();

            } catch (DataValidationException e) {
                JOptionPane.showMessageDialog(this, e.getValidationMessage());
            } catch (ToolException e) {
                JOptionPane.showMessageDialog(this, e.getMessage());
                e.printStackTrace(System.err);
            }
        }
    }

    // override InnerPanel
    protected Component[] getFirstFocusComponents() {
        Component[] comps = new Component[0];
        Component[] genComps = new Component[0];

        genComps = generalPanel.getFirstFocusComponents();
        comps = new Component[genComps.length + 1];
        comps[0] = tab;
        for (int i = 0; i < genComps.length; i++) {
            comps[i + 1] = genComps[i];
        }
        return comps;
    }

    // override InnerPanel
    public void save() {
        if (getProject() == null) {
            System.err.println("DodsInnerPanel.save() - project null");
        } else {
            try {
                trySave();
            } catch (DataValidationException e) {
                System.err.println(e.getValidationMessage());
            } catch (ToolException e) {
                System.err.println(e.getMessage());
            }
        }
    }

    private void trySave() throws DataValidationException, ToolException {
        try {
            write(getProject());
        } catch (DataValidationException e) {
        }
    }

    //
    private void pmInit() {
        generalPanel = new GeneralPanel();
        addInstructor(generalPanel);
    }

    private void addInstructor(Instructor i) {
        InstructPanel ip = null;

        if (i instanceof JPanel) {
            ip = new InstructPanel();
            ip.addInstuctor(i);
            tab.add(ip, i.getTab());
        } else {
            throw new RuntimeException("Instructor not a JPanel: "
                                       + i.getTitle());
        }
    }

    private void jbInit() throws Exception {
        tab = (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                              JTabbedPane.class.getName());
        layoutMain =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        this.setLayout(layoutMain);
        this.add(tab, BorderLayout.CENTER);
    }

    private class TabChangeListener implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            if (!tab.getSelectedComponent().isShowing()) {
                int selectIndex = tab.getSelectedIndex();

                if (selectIndex > showIndex) {
                    tab.setSelectedIndex(showIndex);
                    for (int i = showIndex; i < selectIndex; i++) {
                            break;
                    }
                    showIndex = tab.getSelectedIndex();
                } else if (selectIndex < showIndex) {
                    showIndex = selectIndex;
                }
            }
            refresh();
        }

    }
    private DodsStep[] getSteps() {
        DodsStep[] steps = new DodsStep[1];

        steps[0] = generalPanel;
        return steps;
    }

}
