/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.kelp.common.importer;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;

// Kelp imports
import org.enhydra.kelp.common.map.MapEntry;
import org.enhydra.kelp.common.map.MapUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.node.PropertyKeys;

// Standard imports
import java.io.BufferedReader;
import java.io.IOException;
import java.io.File;
import java.io.FileReader;
import java.util.Vector;

//
public class MakefileReader {

    // strings not to be translated
    private final String   KEEP = "-keep";                // nores
    private final String   IMPLEMENTS = "-implements";    // nores
    private final String   ROOT = "ROOT";                 // nores
    private final String   PACKAGEDIR = "PACKAGEDIR";     // nores
    private final String   XMLC_ = "XMLC_";               // nores
    private final String   _CLASSES = "_CLASSES";         // nores
    private final String   _DIR = "_DIR";                 // nores
    private final String   _OPTS = "_OPTS";               // nores
    private final String   _OPTS_FILE = "_OPTS_FILE";     // nores
    private final String   PE = "+=";                     // nores

    //
    private OtterProject   project = null;
    private MakefileFilter filter = null;
    private MakefileData   makeCursor = null;
    private MakefileData[] makeData = new MakefileData[0];
    private Vector         makeVector = new Vector();
    private String[]       docTypes = new String[0];
    private String         sourcePath = new String();
    private String         continueName = new String();
    private boolean        continueLine = false;
    private boolean        needRead = true;
    private boolean        keepFound = false;

    public MakefileReader() {
        filter = new MakefileFilter();
        docTypes = ToolBoxInfo.getSupportedDocTypes();
    }

    public OtterProject getProject() {
        return project;
    }

    public void setProject(OtterProject p) {
        project = p;
    }

    public void invalidate() {
        needRead = true;
    }

    public void read() {
        File     root = null;
        String[] sourcePaths = getProject().getSourcePathArray();

        for (int i = 0; i < sourcePaths.length; i++) {
            sourcePath = sourcePaths[i];
            root = new File(sourcePath);
            readDirectory(root);
        }
        makeData = new MakefileData[makeVector.size()];
        makeData = (MakefileData[]) makeVector.toArray(makeData);
        makeVector.clear();
        makeCursor = null;
        needRead = false;
    }

    public String[][] getPackageMap() {
        String[][] map = new String[0][2];
        String[]   docs = new String[0];
        String     javaPackage = new String();
        Vector     mapVector = new Vector();
        MapEntry[] entries = new MapEntry[0];
        PathHandle path = null;

        if (needRead) {
            read();
        }
        for (int i = 0; i < makeData.length; i++) {
            javaPackage = makeData[i].getJavaPackage();
            docs = makeData[i].getSourceDocPaths();
            for (int j = 0; j < docs.length; j++) {
                MapEntry entry = null;

                path = PathHandle.createPathHandle(docs[j]);
                path = path.getParent();
                entry = new MapEntry(path.getPath(), javaPackage);
                if (!mapVector.contains(entry)) {
                    mapVector.addElement(entry);
                }
            }
        }
        mapVector.trimToSize();
        entries = new MapEntry[mapVector.size()];
        entries = (MapEntry[]) mapVector.toArray(entries);
        entries = MapUtil.optimize(entries);
        map = MapUtil.toStringArray(entries);
        entries = new MapEntry[0];
        mapVector.clear();
        return map;
    }

    public MakefileData[] getSymbolicMakefiles() {
        MakefileData[] syms = new MakefileData[0];
        Vector         symVector = new Vector();

        if (needRead) {
            read();
        }
        for (int i = 0; i < makeData.length; i++) {
            if (makeData[i].isSymbolic()) {
                symVector.addElement(makeData[i]);
            }
        }
        syms = new MakefileData[symVector.size()];
        syms = (MakefileData[]) symVector.toArray(syms);
        symVector.clear();
        return syms;
    }

    public void updateProject() {
        if (needRead) {
            read();
        }
        for (int i = 0; i < makeData.length; i++) {
            makeCursor = makeData[i];
            setupXMLCNodes();
        }

        // Generate files into source path if keep found.
        if (keepFound) {
            project.setProperty(PropertyKeys.NAME_GENTO, (new String()) + 0);
        }

        // Turn of auto package if class output is within
        // a source path
        String[]   src = new String[0];
        String     out = new String();
        PathHandle outPath = null;

        src = project.getSourcePathArray();
        out = project.getClassOutputPath();
        outPath = PathHandle.createPathHandle(out);
        for (int i = 0; i < src.length; i++) {
            PathHandle srcCursor = null;

            srcCursor = PathHandle.createPathHandle(src[i]);
            if (srcCursor.parentOf(outPath)) {
                project.setProperty(PropertyKeys.NAME_AUTO_PACK,
                                    Boolean.FALSE.toString());
                break;
            }
        }
    }

    private void readDirectory(File root) {
        File[] list = new File[0];

        list = root.listFiles(filter);
        if (list != null) {
            for (int i = 0; i < list.length; i++) {
                if (list[i].isDirectory()) {
                    readDirectory(list[i]);
                } else {
                    readFile(list[i]);
                }
            }
        }
    }

    private void setupXMLCNodes() {
        OtterXMLCNode nodes[] = getProject().getAllXMLCNodes();

        for (int i = 0; i < nodes.length; i++) {
            setupNodeSelection(nodes[i]);
            setupNodeOption(nodes[i]);
        }
    }

    private void setupNodeSelection(OtterXMLCNode node) {
        String[]   docPaths = makeCursor.getSourceDocPaths();
        String     selectPath = new String();
        String     optionFile = new String();
        String     parameters = new String();
        PathHandle nodePath = null;

        nodePath = PathHandle.createPathHandle(node.getFilePath());
        for (int i = 0; i < docPaths.length; i++) {
            if (nodePath.equals(docPaths[i])) {
                node.setSelected(true);
                optionFile = makeCursor.getXMLCOptionFilePath();
                parameters = makeCursor.getXMLCParameters();
                if (optionFile.length() > 0) {
                    node.getParent().setXMLCOptionFilePath(optionFile);
                }
                if (parameters.length() > 0) {
                    node.getParent().setXMLCParameters(parameters);
                }
                break;
            }
        }
    }

    private void checkForKeep(String params) {
        if (keepFound || (params == null) || (params.trim().length() == 0)) {

            // done
        } else {
            keepFound = (params.indexOf(KEEP) > -1);
        }
    }

    private void setupNodeOption(OtterXMLCNode node) {
        MakeItemData[] items = makeCursor.getItems();
        PathHandle     docPath = null;
        PathHandle     nodePath = null;
        StringBuffer   pathBuf = new StringBuffer();
        boolean        breakOut = false;

        nodePath = PathHandle.createPathHandle(node.getFilePath());
        for (int i = 0; i < items.length; i++) {
            for (int j = 0; j < docTypes.length; j++) {
                pathBuf.setLength(0);
                pathBuf.append(makeCursor.getDocPath());
                pathBuf.append(File.separator);
                pathBuf.append(items[i].getItem());
                pathBuf.append('.');
                pathBuf.append(docTypes[j].toLowerCase());
                docPath = PathHandle.createPathHandle(pathBuf.toString());
                if (docPath.equals(nodePath)) {
                    node.setXMLCParameters(items[i].getOptionList());
                    breakOut = true;
                    break;
                }
            }
            if (breakOut) {
                break;
            }
        }
    }

    private void readFile(File make) {
        makeCursor = new MakefileData(make.getParent(), sourcePath);
        BufferedReader reader = null;

        try {
            reader = new BufferedReader(new FileReader(make));
            String line = reader.readLine();

            while (line != null) {
                readLine(line);
                line = reader.readLine();
            }
            reader.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        if (makeCursor.isXMLC()) {
            makeVector.addElement(makeCursor);
        }
    }

    private void readLine(String in) {
        String line = new String(in.trim());
        String name = new String();
        String value = new String();
        int    index = -1;

        if (line.length() > 0) {
            index = line.indexOf('=');
            if (index > -1) {
                if (line.indexOf(PE) > -1) {    // +=
                    name = line.substring(0, index - 1).trim();
                    value = line.substring(index + 1).trim();
                } else {
                    name = line.substring(0, index).trim();
                    value = line.substring(index + 1).trim();
                }
            } else if (continueLine) {
                name = continueName;
                value = line;
            }
            name = trimContinue(name);
            value = trimContinue(value);
            value = preprocessValue(value);
            if (name.trim().length() == 0 || value.trim().length() == 0) {

                // empty
            } else if (name.equals(ROOT)) {
                makeCursor.readRootPath(value);
            } else if (name.equals(PACKAGEDIR)) {
                makeCursor.readPackagePath(value);
            } else {
                boolean keyFound = false;

                for (int i = 0; i < docTypes.length; i++) {
                    String type = docTypes[i].toUpperCase().trim();

                    if (name.equals(XMLC_ + type + _OPTS_FILE)) {
                        makeCursor.readXMLCOptionFilePath(value);
                        keyFound = true;
                        break;
                    } else if (name.equals(XMLC_ + type + _OPTS)) {
                        checkForKeep(value);
                        makeCursor.setXMLCParameters(value);
                        keyFound = true;
                        break;
                    } else if (name.equals(type + _DIR)) {
                        makeCursor.readDocPath(value);
                        keyFound = true;
                        break;
                    } else if (name.equals(type + _CLASSES)) {
                        makeCursor.addSourceDocPath(value);
                        keyFound = true;
                        break;
                    }
                }
                if (!keyFound) {
                    if (name.startsWith(XMLC_) && name.endsWith(_OPTS)) {
                        String item = name.substring(XMLC_.length(),
                                                     (name.length()
                                                      - _OPTS.length()));

                        makeCursor.addItem(item, value);
                    }
                }
            }
        }
        continueLine = (line.length() > 0)
                       && (line.charAt(line.length() - 1) == '\\');
        if (name.length() > 0) {
            continueName = name;
        }
    }

    private String preprocessValue(String in) {
        String       out = null;
        String       key = new String();
        String       remainder = new String();
        StringBuffer buf = new StringBuffer();
        int          index = -1;

        if (in == null) {
            out = null;
        } else {
            out = new String(in);
            key = IMPLEMENTS;
            index = out.indexOf(key);
            if (index > -1) {
                remainder = out.substring(index + key.length()).trim();
                if (remainder.indexOf('.') == -1) {
                    buf.append(out.substring(0, index));
                    buf.append(key);
                    buf.append(' ');
                    buf.append(makeCursor.getJavaPackage());
                    buf.append('.');
                    buf.append(remainder);
                    out = buf.toString();
                }
            }
        }
        return out;
    }

    private String trimContinue(String in) {
        String out = null;

        if (in != null) {
            out = new String(in);
            while ((out.length() > 0)
                   && (out.charAt(out.length() - 1) == '\\')) {
                out = out.substring(0, out.length() - 1).trim();
            }
        }
        return out;
    }

}
