
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ResUtil;

// JDK
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.jar.Attributes;
import java.util.jar.JarEntry;
import java.util.jar.JarOutputStream;
import java.util.jar.Manifest;
import java.util.ResourceBundle;

//
public class JarBuilder implements Constants, ManifestKeys {

    // not to be resourced
    static ResourceBundle   res =
        ResourceBundle.getBundle("org.enhydra.tool.archive.Res");    // nores

    //
    private JarPlan         plan = null;
    private JarOutputStream jarStream = null;
    private String          archivePath = new String();

    static public void main(String[] args) {
        JarPlan    plan = new JarPlan();
        JarBuilder builder = new JarBuilder();

        try {
            plan.setArchivePath("d:\\temp\\test.jar");
            plan.setClassRoot("d:\\tmp");
            plan.setManifestPath("d:\\tmp\\META-INF\\MANIFEST.MF");
            builder.setPlan(plan);
            builder.buildArchive();
        } catch (ArchiveException e) {
            e.printStackTrace();
        }
    }

    public JarPlan getPlan() {
        return plan;
    }

    public void setPlan(JarPlan p) {
        plan = p;
    }

    public File buildArchive() throws ArchiveException {
        File         jar = null;
        String       root = null;
        File[]       files = new File[0];
        Descriptor[] dd = new Descriptor[0];

        jar = openJarStream(getPlan().getArchivePath());
        root = getPlan().getClassRoot();
        dd = getPlan().getDescriptors();
        files = getPlan().getClassFileArray();
        addJarClasses();
        for (int i = 0; i < dd.length; i++) {
            addDescriptor(dd[i], root, files);
        }
        closeJarStream();
        return jar;
    }

    private Manifest readManifest() throws ArchiveException {
        Manifest            manifest = null;
        FileInputStream     fileStream = null;
        BufferedInputStream bufStream = null;

        try {
            fileStream = new FileInputStream(plan.getManifestPath());
            bufStream = new BufferedInputStream(fileStream);
            manifest = new Manifest(bufStream);
        } catch (FileNotFoundException e) {
            throw new ArchiveException(e,
                                       "Unable to read manifest: "
                                       + plan.getManifestPath());
        } catch (IOException e) {
            throw new ArchiveException(e,
                                       "Unable to read manifest: "
                                       + plan.getManifestPath());
        }
        return manifest;
    }

    protected Manifest buildManifest(Manifest m) {
        Manifest     manifest = m;
        String[]     libs = new String[0];
        StringBuffer buf = new StringBuffer();
        Descriptor[] dd = new Descriptor[0];

        manifest.getMainAttributes().put(new Attributes.Name(MANIFEST_VERSION),
                                         VERSION_VALUE);
        buf.append(System.getProperty(JAVA_VERSION));
        buf.append(' ');
        buf.append('(');
        buf.append(System.getProperty(JAVA_VENDOR));
        buf.append(')');
        manifest.getMainAttributes().put(new Attributes.Name(CREATED_BY),
                                         buf.toString());
        manifest.getMainAttributes().put(new Attributes.Name(KELP_VERSION),
                                         ToolBoxInfo.getToolBoxVersion());
        if (getPlan().getClassRoot() != null) {
            manifest.getMainAttributes().put(new Attributes.Name(CLASS_ROOT),
                                             getPlan().getClassRoot());
        }

        // lib
        libs = getPlan().getLibFiles();
        for (int i = 0; i < libs.length; i++) {
            manifest.getMainAttributes().put(new Attributes.Name(LIBRARY + i),
                                             libs[i]);
        }

        // deployment descriptors
        dd = getPlan().getDescriptors();
        for (int i = 0; i < dd.length; i++) {
            if (dd[i].isValid()) {
                manifest.getMainAttributes().put(new Attributes.Name(dd[i].getType()),
                                                 dd[i].getPath());
            }
        }
        return manifest;
    }

    protected void closeJarStream() throws ArchiveException {
        try {
            jarStream.flush();
            jarStream.finish();
            jarStream.close();
        } catch (IOException e) {
            throw new ArchiveException(e, res.getString("Unable_to_close"));
        }
    }

    protected File openJarStream(String p) throws ArchiveException {
        FileOutputStream fileStream = null;
        File             archiveFile = null;
        Manifest         manifest = null;

        archivePath = p;
        archiveFile = new File(archivePath);
        archivePath = archiveFile.getAbsolutePath();
        verifyArchiveFile(archiveFile);
        try {
            fileStream = new FileOutputStream(archiveFile);
            if (plan.getManifestPath() == null) {
                manifest = buildManifest(new Manifest());
            } else {
                manifest = readManifest();
            }
            jarStream = new JarOutputStream(fileStream, manifest);
        } catch (FileNotFoundException e) {
            throw new ArchiveException(e, res.getString("Unable_to_create"));
        } catch (IOException e) {
            throw new ArchiveException(e, res.getString("Unable_to_create"));
        }
        return archiveFile;
    }

    protected void createEntry(File source, String entryPath)
            throws FileNotFoundException, IOException {
        if (source.getAbsolutePath().equalsIgnoreCase(archivePath)) {

            // ignore recursive entries.
        } else {
            BufferedInputStream in = null;

            in = new BufferedInputStream(new FileInputStream(source));
            createEntry(in, entryPath);
        }
    }

    protected void createEntry(InputStream source, String entryPath)
            throws FileNotFoundException, IOException {
        JarEntry entry = null;

        entry = new JarEntry(entryPath);
        entry.setTime(System.currentTimeMillis());
        jarStream.putNextEntry(entry);

        FileUtil.copy(source, jarStream);
        jarStream.flush();
        jarStream.closeEntry();
        source.close();
    }

    protected void addJarClasses() throws ArchiveException {
        File[] files = new File[0];

        files = getPlan().getClassFileArray();
        for (int i = 0; i < files.length; i++) {
            addJarClassesFile(files[i]);
        }
    }

    //
    //
    private void verifyArchiveFile(File archiveFile) throws ArchiveException {
        if (archiveFile.isDirectory()) {
            String exMess = res.getString("Target_is_a_directory");

            exMess = ResUtil.format(exMess, archiveFile);
            throw new ArchiveException(exMess);
        } else if (archiveFile.getParentFile() == null) {
            String exMess = res.getString("Target_has_invalid");

            exMess = ResUtil.format(exMess, archiveFile);
            throw new ArchiveException(exMess);
        } else {
            archiveFile.getParentFile().mkdirs();
        }
    }

    private void addJarClassesFile(File source) throws ArchiveException {
        StringBuffer entryPath = new StringBuffer();
        String       classRoot = null;
        String       relative = null;

        //
        classRoot = getPlan().getClassRoot();
        relative = source.getAbsolutePath().substring(classRoot.length() + 1);
        relative = relative.replace('\\', '/');

        //
        entryPath.append(relative);

        //
        try {
            createEntry(source, entryPath.toString());
        } catch (IOException e) {
            String exMess = res.getString("JarBuilder_Unable_to");

            exMess = ResUtil.format(exMess, source);
            throw new ArchiveException(e, exMess);
        }
    }

    protected void addDescriptor(Descriptor desc, String root,
                                 File[] files) throws ArchiveException {
        if (isDescriptorFound(desc, root, files)) {

            // do nothing
        } else if (desc.getPath().length() > 0) {
            try {
                createEntry(desc.getInputStream(), desc.getArchivePath());
            } catch (IOException e) {
                throw new ArchiveException(e,
                                           "Unable to add " + desc.getType()
                                           + " descriptor: "
                                           + desc.getPath());
            }
        }
    }

    private boolean isDescriptorFound(Descriptor dd, String root,
                                      File[] files) {
        boolean      in = false;
        StringBuffer jarPath = new StringBuffer();
        String       relative = null;

        for (int i = 0; i < files.length; i++) {
            relative = files[i].getAbsolutePath().substring(root.length()
                    + 1);
            relative = relative.replace('\\', '/');
            if (relative.equals(dd.getArchivePath())) {
                in = true;
                break;
            }
        }
        return in;
    }

}
