
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.common;

// ToolBox
import org.enhydra.tool.common.event.FirstFocusEvent;
import org.enhydra.tool.common.event.FirstFocusListener;

// JDK
import java.awt.*;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.beans.*;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Arrays;
import javax.swing.*;

//
public class InnerPanel extends JPanel implements FirstFocusListener {
    private boolean              init = true;
    private boolean              saveOnClose = false;
    private String               progressTitle = new String();
    private FirstFocusListener[] firstListeners = new FirstFocusListener[0];
    private LocalFocusListener   focusListener = null;
    private LocalWindowListener  windowListener = null;

    public static InnerPanel findAncestor(Component comp) {
        Container  ancestor = null;
        InnerPanel found = null;

        ancestor = comp.getParent();
        while (ancestor != null) {
            ancestor = ancestor.getParent();
            if (ancestor instanceof InnerPanel) {
                found = (InnerPanel) ancestor;
                break;
            }
        }
        return found;
    }

    public InnerPanel() {
        super();
    }

    public Window getWindow() {
        Window window = null;

        if (getTopLevelAncestor() instanceof Window) {
            window = (Window) getTopLevelAncestor();
        }
        return window;
    }

    public Dialog getDialog() {
        Dialog dialog = null;

        if (getWindow() instanceof Dialog) {
            dialog = (Dialog) getWindow();
        }
        return dialog;
    }

    public void setSaveOnClose(boolean b) {
        saveOnClose = b;
        if (saveOnClose) {
           preShow();
        }
    }

    public boolean isSaveOnClose() {
        return saveOnClose;
    }

    /**
     * Remove all object references.
     */
    public void clearAll() {
        clearWindowListener();
        clearFocusListener();
        removeAll();
        firstListeners = new FirstFocusListener[0];
    }

    public String getProgressTitle() {
        return progressTitle;
    }

    public void setProgressTitle(String t) {
        progressTitle = t;
    }

    public void initPreferredSize() {
        Dimension d = new Dimension();

        d.height = 342;
        d.width = 542;
        setPreferredSize(d);
    }

    public void save() {
        System.err.println("InnerPanel.save() not implemented");
    }

    public void addFirstFocusListener(FirstFocusListener listener) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(firstListeners));
        if (!list.contains(listener)) {
            list.add(listener);
            list.trimToSize();
            firstListeners = new FirstFocusListener[list.size()];
            firstListeners =
                (FirstFocusListener[]) list.toArray(firstListeners);
        }
        list.clear();
    }

    public void removeFirstFocusListener(FirstFocusListener listener) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(firstListeners));
        if (list.contains(listener)) {
            list.remove(listener);
            list.trimToSize();
            firstListeners = new FirstFocusListener[list.size()];
            firstListeners =
                (FirstFocusListener[]) list.toArray(firstListeners);
        }
        list.clear();
    }

    private void notifyFirstFocusListeners() {
        FirstFocusEvent event = new FirstFocusEvent(this);

        for (int i = 0; i < firstListeners.length; i++) {
            firstListeners[i].onFirstFocus(event);
        }
        firstListeners = new FirstFocusListener[0];
    }

    // protected
    protected Component[] getFirstFocusComponents() {
        Component[] comps = new Component[1];

        comps[0] = this;
        System.err.println("InnerPanel.getFirstFocusComponents() not implemented");
        return comps;
    }

    public void onFirstFocus(FirstFocusEvent event) {
        preShow();
        if (isSaveOnClose()) {
            initWindowListener();
        }
        clearFocusListener();
    }

    public void preShow() {
        Component[] comps = getFirstFocusComponents();
        addFirstFocusListener(this);
        if (focusListener == null) {
            focusListener = new LocalFocusListener();
            for (int i = 0; i < comps.length; i++) {
                comps[i].addFocusListener(focusListener);
            }
        }
    }

    private void clearFocusListener() {
        Component[] comps = getFirstFocusComponents();

        if (focusListener != null) {
            for (int i = 0; i < comps.length; i++) {
                comps[i].removeFocusListener(focusListener);
            }
        }
        focusListener = null;
    }

    private void initWindowListener() {
        if (windowListener == null) {
            if (getWindow() != null) {
                windowListener = new LocalWindowListener();
                getWindow().addWindowListener(windowListener);
            }
        }
    }

    private void clearWindowListener() {
        Window win = null;

        if (windowListener != null) {
            if (getWindow() != null) {
                getWindow().removeWindowListener(windowListener);
            }
        }
        windowListener = null;
    }

    //
    private class LocalFocusListener extends FocusAdapter {
        public void focusGained(FocusEvent e) {
            if (init) {
                init = false;
                notifyFirstFocusListeners();
            }
        }

    }
    private class LocalWindowListener extends WindowAdapter {
        private boolean invoked = false;

        public void windowClosed(WindowEvent event) {
            if (!invoked) {
                saveAndClear(event.getWindow());
            }
        }

        public void windowClosing(WindowEvent event) {
            invoked = true;
            saveAndClear(event.getWindow());
        }

        private void saveAndClear(Window win) {
            save();
            clearAll();
            win.dispose();
        }

    }
}
