
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.common.wizard;

// ToolBox
import org.enhydra.tool.common.event.FirstFocusEvent;
import org.enhydra.tool.common.event.FirstFocusListener;
import org.enhydra.tool.common.DialogHandler;
import org.enhydra.tool.common.ButtonPanel;
import org.enhydra.tool.common.InnerPanel;
import org.enhydra.tool.common.DialogPanel;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.event.HelpEvent;
import org.enhydra.tool.common.event.HelpListener;

// JDK
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.beans.*;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

/**
 * The TBWizardDialog defines a default container for presenting
 * wizards. To use this class you need to a create one or more
 * TBWizardPage objects and add them to this dialog. This dialog
 * supplies action buttons for the Previous, Next, Finish,
 * Cancel and Help.
 * <P><I>
 * This class is used when creating a standalone wizard. Use the
 * OpenTools BasicWizard when developing JBuilder add-ins.
 * </I></P>
 */
abstract public class TBWizard extends DialogHandler
    implements FirstFocusListener {
    //
    static ResourceBundle          res =
        ResourceBundle.getBundle("org.enhydra.tool.common.Res");    // nores

    //

    private TBWizardButtonPanel    buttonPanel;
    private TBWizardInnerPanel     innerPanel;
    private TBWizardButtonListener buttonListener;
    private WeakReference          backRef = null;
    private WeakReference          nextRef = null;
    private WeakReference          finishRef = null;

    /**
     * Create a wizard dialog as a child of another dialog.
     *
     * @param owner
     * The dialog that is the parent of this wizard.
     */
    public TBWizard() {
        super();
        innerPanel = new TBWizardInnerPanel();
        buttonPanel = new TBWizardButtonPanel();
        innerPanel.addFirstFocusListener(this);
    }

    // implments FirstFocusListener
    public void onFirstFocus(FirstFocusEvent event) {
        refreshButtons();
    }

    /**
     * Clear all object references.
     */
    protected void clearAll() {
        super.clearAll();
        if (backRef != null) {
            backRef.clear();
        }
        if (nextRef != null) {
            nextRef.clear();
        }
        if (finishRef != null) {
            finishRef.clear();
        }
        innerPanel = null;
        buttonPanel = null;
        backRef = null;
        nextRef = null;
        finishRef = null;
    }

    public TBWizardDeck getDeck() {
        return innerPanel.getDeck();
    }

    public void setDeck(TBWizardDeck d) {
        innerPanel.setDeck(d);
    }

    // implements DialogHandler
    public ActionListener createButtonListener() {
        return (new TBWizardButtonListener(this));
    }

    // implements DialogHandler
    public ButtonPanel getButtonPanel() {
        return buttonPanel;
    }

    public InnerPanel getInnerPanel() {
        return innerPanel;
    }

    public void setLargeIcon(Icon icon) {
        innerPanel.setLargeIcon(icon);
    }

    public Icon getLargeIcon() {
        return innerPanel.getLargeIcon();
    }

    /**
     * Add a page to the wizard.
     *
     * @param page
     * A panel that contains swing controls that
     * are specific to the current wizard.
     */
    public void addWizardPage(TBWizardPage page) {
        innerPanel.addWizardPage(page);
    }

    public void back() {
        innerPanel.getDeck().back();
        refreshButtons();
    }

    public void next() {
        innerPanel.getDeck().next();
        refreshButtons();
    }

    public void finish() {
        setOption(JOptionPane.OK_OPTION);
        super.closeWindow();
    }

    public void cancel() {
        setOption(JOptionPane.CANCEL_OPTION);
        super.closeWindow();
    }

    //
    //
    private Component getNext() {
        Component comp = null;

        if (nextRef == null) {
            comp = ButtonPanel.findButton(getInnerPanel().getTopLevelAncestor(),
                                          ButtonPanel.COMMAND_NEXT);
            if (comp != null) {
                nextRef = new WeakReference(comp);
            }
        } else {
            comp = (Component) nextRef.get();
        }
        return comp;
    }

    private Component getBack() {
        Component comp = null;

        if (backRef == null) {
            comp = ButtonPanel.findButton(getInnerPanel().getTopLevelAncestor(),
                                          ButtonPanel.COMMAND_BACK);
            if (comp != null) {
                backRef = new WeakReference(comp);
            }
        } else {
            comp = (Component) backRef.get();
        }
        return comp;
    }

    private Component getFinish() {
        Component comp = null;

        if (finishRef == null) {
            comp = ButtonPanel.findButton(getInnerPanel().getTopLevelAncestor(),
                                          ButtonPanel.COMMAND_FINISH);
            if (comp != null) {
                finishRef = new WeakReference(comp);
            }
        } else {
            comp = (Component) finishRef.get();
        }
        return comp;
    }

    public void refreshButtons() {
        Component back = null;
        Component next = null;
        Component finish = null;
        boolean   backEnable = true;
        boolean   nextEnable = true;
        boolean   finishEnable = true;

        back = getBack();
        next = getNext();
        finish = getFinish();
        if (innerPanel.getDeck().isLastPageShowing()) {
            nextEnable = false;
        }
        if (innerPanel.getDeck().isFirstPageShowing()) {
            backEnable = false;
            finishEnable = false;
        }
        if (back != null) {
            back.setEnabled(backEnable);
        }
        if (next != null) {
            next.setEnabled(nextEnable);
        }
        if (finish != null) {
            finish.setEnabled(finishEnable);
        }
    }

}
