
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.codegen;

// Toolbox imports
import org.enhydra.tool.common.ResUtil;
import java.util.ResourceBundle;

/**
 * GeneratorOption define a generator specific option for CodeGen. Generation
 * options include information such as the directory where to generate
 * an application and name to use for the generated application.
 */
public class GeneratorOption {
    static ResourceBundle res = ResourceBundle.getBundle("org.enhydra.tool.codegen.Res"); // nores

    private String  name = new String();
    private String  value = new String();
    private String  displayName = new String();
    private String  description = new String();
    private boolean required = false;
    private boolean persistent = true;
    private boolean bool = false;

    /**
     * Create a option for the current generator. Generators
     * normally instantiate an array of options during
     * initialization.
     *
     * @param name
     * When passing command line arguments, the name must be prefixed
     * with a dash.
     *
     * @param name
     * When passing command line arguments, the name must be prefixed
     * with a dash.
     *
     *
     * @param value
     * When the name refers to a boolean flag, any value other than 'true'
     * should be treated as 'false'.
     *
     * @param description
     * The description is what displays when help for the option is
     * printed out from the command line. The description may also be
     * used in a wizard panel.
     *
     * @param required
     * True if a value for this option must be specified for the
     * generation to be successful.
     *
     * @throws GeneratorException
     * Thrown if name is a null or an empty string. Note that no exception
     * is thrown during construction if required is true and the value is
     * null or empty.
     */
    public GeneratorOption(String name, String value, String displayName,
                           String description, boolean required,
                           boolean persistent) throws GeneratorException {
        if (name == null || name.trim().length() == 0) {
            throw new GeneratorException(res.getString("Option_name_cannot_be"));
        } else {
            this.name = name;
            this.value = value;
            this.displayName = displayName;
            this.description = description;
            this.required = required;
            this.persistent = persistent;
            this.bool = false;
        }
    }

    /**
     * Create a option for the current generator. Generators
     * normally instantiate an array of options during
     * initialization.
     *
     * @param name
     * When passing command line arguments, the name must be prefixed
     * with a dash.
     *
     * @param value
     * When the name refers to a boolean flag, any value other than 'true'
     * should be treated as 'false'.
     *
     * @param description
     * The description is what displays when help for the option is
     * printed out from the command line. The description may also be
     * used in a wizard panel.
     *
     *
     * @throws GeneratorException
     * Thrown if name is a null or an empty string. Note that no exception
     * is thrown during construction if required is true and the value is
     * null or empty.
     */
    public GeneratorOption(String n, boolean v, String desc,
                           boolean persist) throws GeneratorException {
        this(n, (new Boolean(v)).toString(), new String(), desc, false,
             persist);
        this.bool = true;
    }

    /**
     * Get the option name. This should be a terse name used for command
     * line and referencing within generator code. This terse format of
     * indicating the option should not be displayed in a wizard.
     *
     * @return
     * The option name. Prefix this with a dash for command line usage.
     */
    public String getName() {
        return name;
    }

    /**
     * Get the option value.
     *
     * @return
     * For boolean flags, treat any value other than 'true' as 'false'.
     */
    public String getValue() {
        return value;
    }

    /**
     * Set the option value. The value can be set when processing
     * arguments from the command line or during the writeOptions()
     * method of an CodeGenPanel.
     *
     * @param value
     * String value for the option.
     *
     * @throws GeneratorException
     * Thrown if value is null or empty and required is true.
     */
    public void setValue(String value) throws GeneratorException {
        if (isRequired()) {
            if (value == null || value.trim().length() == 0) {
                throw new GeneratorException(
                  ResUtil.format(res.getString("Required_value_cannot"),
                                             getDisplayName()));
            }
        }
        this.value = value;
    }

    /**
     * Set the value as a boolean.
     *
     * @param value
     * Boolean value.
     */
    public void setValue(boolean value)  {
        Boolean b = new Boolean(value);

        this.value = b.toString();
    }

    /**
     * Get a string describing the option. The description
     * is displayed by command line help and may be used
     * for wizard panels.
     *
     * @return
     * Descriptive text. String length should not be
     * longer than 512 characters.
     */
    public String getDescription() {
        return description;
    }

    /**
     * Flag indicating if the values needs to be set prior
     * to generation. Ignore this flag when dealing with
     * boolean generation options.
     *
     * @return
     * If true, the generator requires that the value not be null
     * or an empty string.
     *
     */
    public boolean isRequired() {
        return required;
    }

    /**
     * Check if the option value persistent between CodeGen sessions.
     *
     * @return
     * True if the value is persisted between CodeGen sessions.
     */
    public boolean isPersistent() {
        return persistent;
    }

    /**
     * Get name to display in Swing components.
     *
     * @return
     * String to display in Swing components such as the generator
     * selection combobox.
     *
     */
    public String getDisplayName() {
        return displayName;
    }

    /**
     * Get the value as a boolean.
     *
     * @return
     * Returns true if the value is 'true'.
     */
    public boolean isValue() {
        Boolean b = Boolean.valueOf(getValue().toLowerCase());

        return b.booleanValue();
    }

    /**
     * Determine if the option is a boolean switch.
     *
     * @return
     * True if the option is a boolean.
     */
    public boolean isBoolean() {
        return bool;
    }

    public void clearValue() {
      value = new String();
    }

    public boolean isEmpty() {
      return (value == null || value.trim().length() == 0);
    }

}
