
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.codegen.internal;

// Toolbox imports
import org.enhydra.tool.codegen.wizard.CodeGenPanel;
import org.enhydra.tool.codegen.wizard.ServiceOptionPanel1;
import org.enhydra.tool.codegen.wizard.ProjectOptionPanel2;
import org.enhydra.tool.codegen.wizard.ServiceOptionPanel3;
import org.enhydra.tool.codegen.ProjectGenerator;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.common.Replacement;
import org.enhydra.tool.common.ToolException;

// Standard imports
import java.util.ArrayList;
import java.util.Arrays;

/**
 * A generator for Enhydra 4 services.
 */
public class ServiceGenerator extends ProjectGenerator {

    /**
     * Name to use specify at the command line.
     */
    public static final String COMMAND_NAME = "service";    // nores

    /**
     * Descriptive name for generator.
     */
    public static final String displayName = "Enhydra Service";
    public static final String wizardTitle = "Enhydra Service Wizard";
    private CodeGenPanel[]     panels = null;

    /**
     * Create an instance of the Web Application generator.
     */
    public ServiceGenerator() {
        setReplacementSet(new ServiceReplacementSet());
        setOptionSet(new ServiceOptionSet());
    }

    public CodeGenPanel[] getWizardPanels() throws GeneratorException {

        // don't initialize until called.
        if (panels == null && isSwing()) {
            panels = new CodeGenPanel[3];
            panels[0] = new ServiceOptionPanel1();
            panels[1] = new ProjectOptionPanel2();
            panels[2] = new ServiceOptionPanel3();
            for (int i = 0; i < panels.length; i++) {
                panels[i].setOptionSet(getOptionSet());
                panels[i].readOptionSet();
            }
        }
        return panels;
    }

    /**
     * Implements AppGenerator
     */
    public String[] getOutputExcludes() {
        String[]  ex = new String[0];
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(super.getOutputExcludes()));

        // not to be resourced
        if (!isMBean()) {                 // nores
            list.add("qualfiers.xml");    // nores
            list.add("MBean.java");       // nores
        }
        list.trimToSize();
        ex = new String[list.size()];
        ex = (String[]) list.toArray(ex);
        list.clear();
        return ex;
    }

    protected String[] getInputIncludes() {
        ArrayList list = null;
        String[]  includes = new String[0];

        list = new ArrayList(Arrays.asList(super.getInputIncludes()));
        if (isMBean()) {
            list.add("mbean");
        } else {
            list.add("nombean");
        }
        list.trimToSize();
        includes = new String[list.size()];
        includes = (String[]) list.toArray(includes);
        list.clear();
        return includes;
    }

    // override ProjectGenerator
    public void initReplacementSet() throws GeneratorException {
        super.initReplacementSet();
        try {
            String serviceName = null;

            serviceName =
                getOptionSet().lookup(ServiceOptionSet.SERVICE).getValue();
            getReplacementSet().lookup(ServiceReplacementSet.at_SERVICE_at).setReplaceWith(serviceName);
        } catch (ToolException e) {
            e.printStackTrace();
            throw new GeneratorException(e,
                                         res.getString("Unable_to_init_rep"));
        }
    }

    /**
     * Get short name used in command line. May also be used
     * to locate template directory.
     *
     * @return
     * Command line identifier.
     */
    public String getCommandName() {
        return COMMAND_NAME;
    }

    /**
     * Name to display in command line messages and in the wizard.
     *
     * @return
     * Display name for generator.
     */
    public String getDisplayName() {
        return ServiceGenerator.displayName;
    }

    public String getWizardTitle() {
        return ServiceGenerator.wizardTitle;
    }

    /**
     * Get text that breifly explains the application that
     * the generator creates.
     *
     * @return
     * Descriptive text to display in a wizard or in a command
     * line help response.
     */
    public String getDescription() {
        return "Generate a simple service that extends the "
               + "Enhydra application server. Refer to the service developer's "
               + "guide for a detailed explaintion of how to implement an "
               + "Enhydra service.";
    }

    protected String[] getDefaultAddinSteps() {
        String[] steps = new String[5];

        steps[0] = "Build your project to compile all source files";
        steps[1] = "Start the Enhydra Multiserver from the command line:<BR>&nbsp;&nbsp;<code>$ cd <i>enhydra root</i>/bin</code><BR>&nbsp;&nbsp;<code>$ ./multiserver</code>";
        steps[2] = "Deploy the project by selecting Tools | Kelp Deployer.<BR> Click the Deploy button to create an archive and deploy it to the running multiserver.";
        steps[3] = "Verify that the service has been successfully deployed by inspecting the multiserver console output.<BR><BR>";
        steps[4] = getCodeExampleStep();
        return steps;
    }

    protected String[] getDefaultShellSteps() {
        String[]     steps = new String[3];
        StringBuffer buf = new StringBuffer();

        buf.append("Use &lt;enhydra_root&gt;/bin/ant to compile your project");
        buf.append(" using the build.xml file.");
        steps[0] = buf.toString();
        buf.setLength(0);
        buf.append("To deploy your service, copy the jar from output/lib");
        buf.append(" to &lt;enhydra_root&gt;/deploy");
        steps[1] = buf.toString();
        steps[2] = getCodeExampleStep();
        return steps;
    }

    private String getCodeExampleStep() {
        StringBuffer buf = new StringBuffer();
        String       pack = new String();
        String       service = new String();

        try {
            pack =
                getReplacementSet().lookup(ServiceReplacementSet.JAVA_PACKAGE).getReplaceWith()[0];
            service =
                getReplacementSet().lookup(ServiceReplacementSet.at_SERVICE_at).getReplaceWith()[0];
        } catch (ToolException e) {
            e.printStackTrace(System.err);
            pack = "&lt;package_name&gt;";
            service = "&lt;service_name&gt;";
        }
        buf.append("To access your new service from a client");
        buf.append(" application:");
        buf.append("<code><br>");
        buf.append("<br>javax.naming.Context      ctx = new javax.naming.InitialContext();");
        buf.append("<br>Object                    obj = ctx.lookup(\"ems:/Services/"
                   + service + "/manager\");");
        buf.append("<br>" + pack + '.' + service + "Target target = (" + pack
                   + '.' + service + "Target) obj;");
        buf.append("<br>String                    greeting = target.getGreeting();");
        buf.append("<br></code>");
        return buf.toString();
    }

    private boolean isMBean() {
        boolean mbean = true;

        try {
            mbean = getOptionSet().lookup(ServiceOptionSet.MBEAN).isValue();
        } catch (GeneratorException e) {
            e.printStackTrace();
            mbean = true;
        }
        return mbean;
    }

}
