
/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.codegen.wizard;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.wizard.TBWizard;
import org.enhydra.tool.codegen.CodeGen;
import org.enhydra.tool.codegen.Generator;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.codegen.ValidationException;

//
// Standard imports
//
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.beans.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

/**
 * The CodeGenDialog defines a default container for presenting
 * wizards. To use this class you need to a create one or more
 * CodeGenPage objects and add them to this dialog. This dialog
 * supplies action buttons for the Previous, Next, Finish,
 * Cancel and Help.
 * <P><I>
 * This class is used when creating a standalone wizard. Use the
 * OpenTools BasicWizard when developing JBuilder add-ins.
 * </I></P>
 */
public class CodeGenWizard extends TBWizard implements RootEditor {

    //
    private static ResourceBundle  cgRes =
        ResourceBundle.getBundle("org.enhydra.tool.codegen.Res");    // nores
    private final static String    CODEGEN_TITLE = "Kelp Application Wizard";

    //
    private File[]                 files = new File[0];
    private GenSelectionPanel      selectionPanel = null;
    private CodeGen                codeGen = null;
    private GenSelectionListener[] selectionListeners =
        new GenSelectionListener[0];

    public static void main(String[] args) {
        CodeGenWizard wizard = null;

        try {
            wizard = new CodeGenWizard();
            wizard.showDialog(null);
        } catch (GeneratorException e) {
            e.printStackTrace(System.err);
        }
        System.exit(0);
    }

    public CodeGenWizard(CodeGen cg) {
        super();
        codeGenInit();
        setCodeGen(cg);
    }

    public CodeGenWizard() throws GeneratorException {
        super();
        codeGenInit();
        setCodeGen(new CodeGen());
    }

    // implements RootEditor
    public boolean isAllowRootEdit() {
        return getCodeGenDeck().isEnabled();
    }

    // implements RootEditor
    public void setAllowRootEdit(boolean allow) {
        getCodeGenDeck().setAllowRootEdit(allow);
    }

    /**
     * Get an array of references to generated files.
     *
     * @return
     * An array of generated files. May have a zero length or
     * be null if no files have been generated.
     *
     */
    public File[] getGeneratedFiles() {
        return files;
    }

    /**
     * Get the list of available generators.
     *
     * @return
     * An array of generators from which a user can make a selection.
     */
    public Generator[] getGenerators() {
        return codeGen.getGenerators();
    }

    /**
     * Set the list of available generators.
     *
     * @param generators
     * An array of generators from which a user can make a selection.
     */
    public void setCodeGen(CodeGen cd) {
        codeGen = cd;
        selectionPanel.setGenerators(codeGen.getGenerators());
    }

    public CodeGen getCodeGen() {
        return codeGen;
    }

    /**
     * Get the generator selected by the user.
     *
     * @return
     * The generator to show options for and to use for generating a project.
     */
    public Generator getSelection() {
        return getCodeGenDeck().getSelection();
    }

    /**
     * Set the selected generator for a user and by pass the
     * selection page.
     *
     * @param selection
     * A generator containing option pages.
     *
     * @exception GeneratorException
     * Thrown if unable to create option pages for the selection.
     *
     */
    public void setSelection(Generator selection) throws GeneratorException {
        getCodeGenDeck().setSelection(selection);
        if (selection != null) {
            getCodeGenDeck().setSelectionPageHidden(true);
            getInnerPanel().getDialog().setTitle(selection.getWizardTitle());
        }
    }

    // overrides TBWizard
    public void back() {
        if (getCodeGenDeck().isSelectionPageHidden()) {

            // nothing special
        } else if (getCodeGenDeck().getPageIndex() == 1) {
            try {
                Generator select = getCodeGenDeck().getSelection();
                setCodeGen(new CodeGen());
                selectionPanel.setSelection(select);
            } catch (GeneratorException e) {
                e.printStackTrace(System.err);
            }
        }
        super.back();
    }

    // overrides TBWizard
    public void next() {
        if (getCodeGenDeck().isSelectionPageShowing()) {
            try {
                Generator gen = selectionPanel.getSelection();

                codeGen.setSelection(gen);
                notifySelectionListeners(gen);

                // listeners my re-instatiate codegen with new selection;
                gen = codeGen.getSelection();
                getCodeGenDeck().setSelection(gen);
                super.next();
            } catch (GeneratorException e) {
                JOptionPane.showMessageDialog(getInnerPanel(),
                                              e.getMessage(),
                                              cgRes.getString("Generator_Error"),
                                              JOptionPane.ERROR_MESSAGE);
                if (CodeGen.debug) {
                    e.printStackTrace();
                }
            }
        } else {
            super.next();
        }
    }

    // overrides TBWizard
    public void finish() {
        Generator selection;

        try {
            getCodeGenDeck().writeOptionSet();
            selection = selectionPanel.getSelection();
            files = selection.generate();
            super.finish();
        } catch (ValidationException e) {
            JOptionPane.showMessageDialog(getInnerPanel(),
                                          e.getValidationMessage(),
                                          cgRes.getString("Validation_Error"),
                                          JOptionPane.ERROR_MESSAGE);
        } catch (GeneratorException e) {
            JOptionPane.showMessageDialog(getInnerPanel(), e.getMessage(),
                                          cgRes.getString("Generation_Error"),
                                          JOptionPane.ERROR_MESSAGE);
        }
    }

    public static String getDefaultTitle() {
        return CODEGEN_TITLE;
    }

    // override TBWizard
    protected void clearAll() {
        super.clearAll();
        selectionPanel.removeAll();
        selectionListeners = new GenSelectionListener[0];
        selectionPanel = null;
        codeGen = null;

        // don't clear files
    }

    // implements TBWizard
    public String getTitle() {
        return CodeGenWizard.getDefaultTitle();
    }

    // implements TBWizard
    protected String getProgressTitle() {
        return "Component Generation Progress";
    }

    //
    public synchronized void addSelectionListener(GenSelectionListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(selectionListeners));
        if (!list.contains(l)) {
            list.add(l);
            list.trimToSize();
            selectionListeners = new GenSelectionListener[list.size()];
            selectionListeners =
                (GenSelectionListener[]) list.toArray(selectionListeners);
        }
        list.clear();
    }

    public synchronized void removeSelecitonListener(GenSelectionListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(selectionListeners));
        if (list.contains(l)) {
            list.remove(l);
            list.trimToSize();
            selectionListeners = new GenSelectionListener[list.size()];
            selectionListeners =
                (GenSelectionListener[]) list.toArray(selectionListeners);
        }
        list.clear();
    }

    private void notifySelectionListeners(Generator gen) {
        GenSelectionEvent event = null;

        event = new GenSelectionEvent(this, gen);
        for (int i = 0; i < selectionListeners.length; i++) {
            selectionListeners[i].onSelection(event);
        }
    }

    /**
     * Add listeners and other items that are not generated by
     * the JBuilder designer.
     */
    private void codeGenInit() {
        CodeGenDeck newDeck = null;
        CodeGenPage page = null;

        newDeck = new CodeGenDeck();
        selectionPanel = new GenSelectionPanel();
        setDeck(newDeck);
        page = new CodeGenPage();
        page.setPageTitle(selectionPanel.getPageTitle());
        page.setInstructions(selectionPanel.getInstructions());
        page.add(selectionPanel);
        addWizardPage(page);
    }

    private CodeGenDeck getCodeGenDeck() {
        return (CodeGenDeck) getDeck();
    }

}
